/*
 * GenericCyclePattern.h
 *
 *  Created on: 02.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_GENERICCYCLEPATTERN_H_
#define FMG_GENERICCYCLEPATTERN_H_

#include <dolfin/common/types.h>

namespace fmg {

/// This class provides an interface for using various cycle patterns
/// used in multigid.

class GenericCyclePattern
{
public:
	typedef struct { enum e { restrict, solve, prolongate, pr_transition }; } Actions;
	typedef Actions::e Action;

	/// Resets the cycle to its initial state,
	/// which usually is action_index=0, cycle_index=0, level_index=0.
	virtual void reset() = 0;

	/// Go to the next action, this also increments action_index,
	/// except if starting a new cycle period action_index is reset to zero.
	virtual void next() = 0;

	/// Return the current action.
	virtual Action action() const = 0;

	/// Return current action index.
	virtual dolfin::uint action_index() const = 0;

	/// Return current cycle index.
	virtual dolfin::uint cycle_index() const = 0;

	/// Return grid level index.
	virtual dolfin::uint level_index() const = 0;
};

} // namespace

#endif /* FMG_GENERICCYCLEPATTERN_H_ */
