/*
 * SmoothingOperator.cpp
 *
 *  Created on: 25.10.2012
 *      Author: Felix Ospald
 */

#include "SmoothingOperator.h"
#include "JacobiSmoothingOperator.h"
#include "SORSmoothingOperator.h"

namespace fmg {

SmoothingOperator::SmoothingOperator(const boost::shared_ptr<const dolfin::GenericMatrix>& A, const std::string& method, double omega)
{
	if (method == "jacobi")
		_S.reset(new JacobiSmoothingOperator(A, omega));
	else if (method == "fsor")
		_S.reset(new SORSmoothingOperator(A, SORSmoothingOperator::SORTypes::forward, omega));
	else if (method == "bsor")
		_S.reset(new SORSmoothingOperator(A, SORSmoothingOperator::SORTypes::backward, omega));
	else if (method == "ssor")
		_S.reset(new SORSmoothingOperator(A, SORSmoothingOperator::SORTypes::symmetric, omega));
	else
		dolfin::error("Unknown smoother '%s'", method.c_str());
}

} // namespace
