/*
 * SmoothingOperator.h
 *
 *  Created on: 25.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_SMOOTHINGOPERATOR_H_
#define FMG_SMOOTHINGOPERATOR_H_

#include <fmg/relax/GenericSmoothingOperator.h>
#include <dolfin/la/GenericMatrix.h>

namespace fmg {

/// Wrapper class for various relaxation methods.

class SmoothingOperator : public GenericSmoothingOperator
{
public:
	/// Constructor. Current valid values for the parameter method are:
	///  * "jacobi": Jacobi relaxation sweep
	///  * "fsor": forward SOR sweep (this is equivalent to Gauss-Seidel if omega=1.0)
	///  * "bsor": backward SOR sweep
	///  * "ssor": symmetric SOR sweep
	SmoothingOperator(const boost::shared_ptr<const dolfin::GenericMatrix>& A, const std::string& method, double omega = 1.0);

	void apply(dolfin::GenericVector& x, const dolfin::GenericVector& b, bool& x_assumed_zero, uint reps) const {
		_S->apply(x, b, x_assumed_zero, reps);
	}

protected:
	boost::shared_ptr<GenericSmoothingOperator> _S;
};

} // namespace


#endif /* FMG_SMOOTHINGOPERATOR_H_ */
