/*
 * MultigridPreconditioner.h
 *
 *  Created on: 01.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_MULTIGRIDPRECONDITIONER_H_
#define FMG_MULTIGRIDPRECONDITIONER_H_

#include <fmg/solve/MultigridSolver.h>

#include <dolfin/la/PETScKrylovSolver.h>
#include <dolfin/la/PETScUserPreconditioner.h>
#include <dolfin/la/PETScVector.h>

namespace fmg {

/// This class implements a PETScUserPreconditioner for multigrid preconditioning.

class MultigridPreconditioner : public dolfin::PETScUserPreconditioner
{
public:
	/// Constructor. Note the passed mg_sover is modified to work as a preconditioner i.e.:
	///
	/// .. code-block:: c
	///
	///   mg_solver.parameters["error_on_nonconvergence"] = false;
	///   mg_solver.parameters["tolerance_checking"] = false;
	///   mg_solver.parameters["maximum_iterations"] = 1;
	///
    /// are set.
	///
	MultigridPreconditioner(MultigridSolver& mg_solver);

	/// shared_ptr version of constructor.
	MultigridPreconditioner(const boost::shared_ptr<MultigridSolver>& mg_solver);

	/// Applies the preconditioner to the rhs b and stores the result to x.
	void solve(dolfin::PETScVector& x, const dolfin::PETScVector& b);

	/// Setup the passed solver to use this preconditioner.
	void setup(dolfin::PETScKrylovSolver& solver);

	/// Return the multigrid solver used as preconditioner.
	MultigridSolver& mg_solver() const { return *_mg_solver; }

protected:
	void init(const boost::shared_ptr<MultigridSolver>& mg_solver);

	boost::shared_ptr<MultigridSolver> _mg_solver;

}; // class

} // namespace

#endif /* FMG_MULTIGRIDPRECONDITIONER_H_ */
